   wire       bus_clk;

   // Wires related to /dev/xillybus_read_32
   wire       user_r_read_32_rden;
   wire       user_r_read_32_empty;
   wire [31:0] user_r_read_32_data;
   wire        user_r_read_32_eof;
   wire        user_r_read_32_open;

   // Wires and registers related to data capturing
   wire        capture_clk;
   reg [31:0]  capture_data;
   wire        capture_en;
   reg [4:0]   slowdown;
   wire        capture_full;

   reg 	       capture_open;
   reg 	       capture_open_cross;
   reg 	       capture_has_been_full;
   reg 	       capture_has_been_nonfull;
   reg 	       has_been_full_cross;
   reg 	       has_been_full;
	     

   always @(posedge capture_clk)
     begin
	if (capture_en)
	  capture_data <= capture_data + 1; // Bogus data source
	
	// The slowdown register limits the data pace to 1/32 the bus_clk
	// when capture_clk = bus_clk. This may be necessary, because the
	// core in the evaluation kit is configured for simplicity, and
	// not for performance.
	// The slowdown register has no function in a real-life application.
	slowdown <= slowdown + 1;

	// capture_has_been_full remembers that the FIFO has been full
	// until the file is closed. capture_has_been_nonfull prevents
	// capture_has_been_full to respond to the initial full condition
	// every FIFO displays on reset.

	if (!capture_full)
	  capture_has_been_nonfull <= 1;
	else if (!capture_open)
	  capture_has_been_nonfull <= 0;
	
	if (capture_full && capture_has_been_nonfull)
	  capture_has_been_full <= 1;
	else if (!capture_open)
	  capture_has_been_full <= 0;
	
     end

   // The dependency on slowdown is only for bogus data
   assign capture_en = capture_open && !capture_full && 
		       !capture_has_been_full &&
		       (slowdown == 0);
      
   // Clock crossing logic: bus_clk -> capture_clk
   always @(posedge capture_clk)
     begin
	capture_open_cross <= user_r_read_32_open;
	capture_open <= capture_open_cross;
     end

   // Clock crossing logic: capture_clk -> bus_clk
   always @(posedge bus_clk)
     begin
	has_been_full_cross <= capture_has_been_full;
	has_been_full <= has_been_full_cross;
     end
   
   // The user_r_read_32_eof signal is required to go from '0' to '1' only on
   // a clock cycle following an asserted read enable, according to Xillybus'
   // core API. This is assured, since it's a logical AND between
   // user_r_read_32_empty and has_been_full. has_been_full goes high when the
   // FIFO is full, so it's guaranteed that user_r_read_32_empty is low when
   // that happens. On the other hand, user_r_read_32_empty is a FIFO's empty
   // signal, which naturally meets the requirement.
   
   assign user_r_read_32_eof = user_r_read_32_empty && has_been_full;
   
   async_fifo_32 fifo_32 
     (
      .rst(!user_r_read_32_open),
      .wr_clk(capture_clk),
      .rd_clk(bus_clk),
      .din(capture_data),
      .wr_en(capture_en),
      .rd_en(user_r_read_32_rden),
      .dout(user_r_read_32_data),
      .full(capture_full),
      .empty(user_r_read_32_empty)
      );
